/*==========================================================================*/
/* MsgTestConv.c contains data conversion related routines                  */
/*==========================================================================*/
/* (C) Copyright Tim Armstrong 2003, 2004, 2005, 2006, 2007, 2008           */
/*==========================================================================*/
/* Includes                                                                 */
/*==========================================================================*/
#ifndef __TJAINC_MSGTEST_H
#include "MsgTest.h"
#endif /* __TJAINC_MSGTEST_H */

/*==========================================================================*/
/* CCSID Aliases                                                            */
/* Each entry contains the numeric CCSID followed by a series of aliases    */
/* separated by semi-colons ";". NOTE the leading and trailing semi-colons  */
/* must be specified or false or missed matches will occur.     */
/* attempt to match the last value will fail. Blanks, dashes and            */
/* underscores are removed from the character set name obtained by the call */
/* to GetLocaleInfo or nl_langinfo before any comparisons are made thus     */
/* ISO-8859-1 and ISO88590-1 are both considered as matches for IS88591     */
/* which translates to CCSID 819.                                           */
/*                                                                          */
/* The CCSID alias table must be terminated by the entry "EOT"              */
/*                                                                          */
/* You can also use the utility program nlinfo, supplied with this          */
/* supportpac, to get character string representation of character set name */
/* in use in the local environment. Once you have done so then looking at   */
/* the ccsid.tbl and readme.ccs files supplied in the MQ conversion tables  */
/* directory and searching the internet for information on code page        */
/* aliases for example "www.iana.org/assignments/character-sets" you should */
/* be able to determine the CCSID of your code page. If you do extend the   */
/* table below please send an email to me at the contact address supplied   */
/* in the users guide                                                       */
/*                                                                          */
/* MsgTest can only convert between two SBCS code pages                     */
/*                                                                          */
/* The following excerpt from  ccsid.tbl describes how to check for the     */
/* existence of a conversion table                                          */
/* #        In order to convert between two CCSIDs then there must          */
/* #        exist tables of type AAAABBBB.TBL where AAAA and BBBB           */
/* #        are the hex representation of the decimal CodePage ids          */
/* #        corresponding to the CCSIDs that conversion is taking           */
/* #        place between.                                                  */
/*==========================================================================*/
static char* kpszCCSIDAlias[] = {
";037;IBM037;CP037;\0", /* Note leading & trailing semi-colons are required */
";500;IBM500;CP500;\0",
";437;IBM437;CP437;ASCII;USASCII;646;\0",
";819;IBM819;CP819;ISO88591;\0",
";850;IBM850;CP850;\0",
";1208;UTF8;UTF-8;CP1208;\0",
";1252;WINDOWS1252;WIN1252;CP1252;\0",
"EOT\0" /* Table must be terminated with EOT */
};

/*==========================================================================*/
/* Static variables                                                         */
/*==========================================================================*/
static char* kpszEOT =                "EOT\0";


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     GetLocalCCSID                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Get the value of the local CCSID if available use the one specified  */
/*     in the <Control> section otherwise try to determine the value        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     CCSID or 0 if unable to determine a value                            */
/*==========================================================================*/
long
GetLocalCCSID()
{
    char lpszCodePageBuff[CODE_PAGE_BUFF_LEN + 4];
    long lCCSID;

    /* Initialise local variables and buffers */
    memset(lpszCodePageBuff, 0x00, CODE_PAGE_BUFF_LEN + 4);

    /* Now get the character form of the local codepage/codeset setting */
#ifdef WIN32
    GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_IDEFAULTCODEPAGE, lpszCodePageBuff, 12);
#else
    setlocale(LC_CTYPE, "");
    sprintf(lpszCodePageBuff, "%s\0", nl_langinfo(CODESET));
#endif /* WIN32 */

    /* Convert the string just obtained to a CCSID */
    lCCSID = ConvertToCCSID(lpszCodePageBuff);
    if(0 != lCCSID)
    {
        LogMsg(218, lpszCodePageBuff, lCCSID);
    }

    return lCCSID;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ConvertToCCSID                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Attempt to convert the named code page to a CCSID number             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszCodePage Name of the code page to look up                        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     CCSID or 0 if unable to determine a value                            */
/*==========================================================================*/
long
ConvertToCCSID(const char* const kpszCodePage)
{
    char* lpszCurrChar;
    char* lpszEndOfStr;
    char* lpszOutputChar;
    char  lpszConvertBuff[CODE_PAGE_BUFF_LEN + 4];
    long  lIdx;

    /* Initialise local variables and buffers */
    memset(lpszConvertBuff, 0x00, CODE_PAGE_BUFF_LEN + 4);

    /* Now remove any whitespace, dashes or underscores */
    /* converting to upper case at the same time        */
    lpszCurrChar = (char*)kpszCodePage;
    lpszEndOfStr = (char*)kpszCodePage + StrlenNull(kpszCodePage);
    lpszOutputChar = lpszConvertBuff;
    *lpszOutputChar = ';'; /* Add the leading semi-colon */
    lpszOutputChar++;

    while(lpszCurrChar < lpszEndOfStr)
    {
        /* Skip whitespace dashes and underscores */
        if((0 != isspace(*lpszCurrChar)) ||
           ('-' == *lpszCurrChar) ||
           ('_' == *lpszCurrChar))
        {
            lpszCurrChar++;
        }
        else
        {
            *lpszOutputChar = *lpszCurrChar;
            *lpszOutputChar = (char)toupper(*lpszOutputChar);
            lpszOutputChar++;
            lpszCurrChar++;
        }
    }

    /* Now terminate the string for our lookup using strstr */
    memcpy(lpszOutputChar, ";\0", 2);

    /* See if we can find a string containing the alias */
    lIdx = 0;
    while(0 != memcmp(kpszCCSIDAlias[lIdx], kpszEOT, strlen(kpszEOT) + 1))
    {
        if(NULL != strstr(kpszCCSIDAlias[lIdx], lpszConvertBuff))
        {
            /* Reuse the conversion buffer */
            memset(lpszConvertBuff, 0x00, CODE_PAGE_BUFF_LEN + 4);

            /* Copy the CCSID that begins the alias string to the */
            /* buffer for numeric conversion                      */
            lpszCurrChar = kpszCCSIDAlias[lIdx];
            lpszEndOfStr = kpszCCSIDAlias[lIdx] + StrlenNull(kpszCCSIDAlias[lIdx]);
            lpszOutputChar = lpszConvertBuff;
            
            /* Skip the leading semi-colon */
            lpszCurrChar++;

            while((lpszCurrChar < lpszEndOfStr) &&
                  (';' != *lpszCurrChar))
            {
                *lpszOutputChar = *lpszCurrChar;
                lpszOutputChar++;
                lpszCurrChar++;
            }

            /* Now try to convert it to a numeric value */
            return ConvNumFailReturnsZero(lpszConvertBuff);
        }
        /* Try the next alias string */
        lIdx++;
    }

    return 0;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ReverseLong                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Reverse the contents of a long.                                      */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplNum  Pointer to long who's bytes are to be reversed               */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ReverseLong(long *lplNum)
{
    unsigned char *lpByte;

    lpByte = (unsigned char *)lplNum;
    *(lpByte + 3) ^= *lpByte;
    *lpByte ^= *(lpByte + 3);
    *(lpByte + 3) ^= *lpByte;
    *(lpByte + 2) ^= *(lpByte + 1);
    *(lpByte + 1) ^= *(lpByte + 2);
    *(lpByte + 2) ^= *(lpByte + 1);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CheckAndFixRFH2Longs                                                 */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check the encoding and if necessary reverse the numeric fields       */
/*     in the RFH2 header                                                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpoRFH2           Pointer to RFH2 header to be operated on           */
/*     lLocalEncoding    Encoding of the local machine                      */
/*     lCurrentEncoding  The way the RFH2 header is currently encoded       */
/*     lRequiredEncoding How the RFH2 header should be encoded              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CheckAndFixRFH2Longs(MQRFH2* lpoRFH2, long lLocalEncoding, long lCurrentEncoding, long lRequiredEncoding)
{
    long  lRFH2DataLen;
    long  lRFH2WorkLen;
    long  lRFH2FolderLen;
    long* lplFolderLen;
    char* lpszFolderData;
    long  lLocalIntegerEncoding;
    long  lCurrentIntegerEncoding;
    long  lRequiredIntegerEncoding;

    /* Only interested in the integer encoding so mask out the other bits */
    lLocalIntegerEncoding = lLocalEncoding & MQENC_INTEGER_MASK;;
    lCurrentIntegerEncoding = lCurrentEncoding & MQENC_INTEGER_MASK;;
    lRequiredIntegerEncoding = lRequiredEncoding & MQENC_INTEGER_MASK;;

    /* If the RFH2 header integers are already encoded as required then leave it alone */
    if(lCurrentIntegerEncoding == lRequiredIntegerEncoding)
    {
        return;
    }

    /* Extract the RFH2 structure length value */
    lRFH2DataLen = lpoRFH2->StrucLength;
    if(lLocalIntegerEncoding != lCurrentIntegerEncoding)
    {
        ReverseLong(&lRFH2DataLen);
    }

    /* Reverse the integer values in the fixed part of the RFh2 header */
    ReverseLong(&lpoRFH2->Version);
    ReverseLong(&lpoRFH2->StrucLength);
    ReverseLong(&lpoRFH2->Encoding);
    ReverseLong(&lpoRFH2->CodedCharSetId);
    ReverseLong(&lpoRFH2->Flags);
    ReverseLong(&lpoRFH2->NameValueCCSID);

    /* Also fix any folders in the RFH2 header */
    lRFH2WorkLen = lRFH2DataLen  - MQRFH_STRUC_LENGTH_FIXED_2;
    lpszFolderData = (char*)lpoRFH2 + MQRFH_STRUC_LENGTH_FIXED_2;
    while(0 < lRFH2WorkLen)
    {
        /* Extract the RFH2 folder length value */
        lplFolderLen = (long*)lpszFolderData;
        lRFH2FolderLen = *lplFolderLen;
        if(lLocalIntegerEncoding != lCurrentIntegerEncoding)
        {
            ReverseLong(&lRFH2FolderLen);
        }
        /* Reverse the folder length value */
        ReverseLong(lplFolderLen);
        /* Move on to the next folder in the RFH2 header taking into account the length fields */
        lRFH2WorkLen -= lRFH2FolderLen + 4;
        lpszFolderData += lRFH2FolderLen + 4;
    }
}


